using System;
using Upsilon.BaseTypes.Number;

namespace Upsilon.BaseTypes
{
    internal class LuaString : LuaType
    {
        public LuaString(string value)
        {
            Value = value;
        }

        public string Value { get; }
        public override Type Type => Type.String;
        public override object ToCSharpObject()
        {
            return Value;
        }

        public override bool Equals(object obj)
        {
            if (obj == null)
                return false;
            if (!(obj is LuaString s))
                return false;
            return Equals(s);
        }

        public override int GetHashCode()
        {
            return (Value != null ? Value.GetHashCode() : 0);
        }

        public bool Equals(LuaString s)
        {
            if (s == null) return false;
            return string.Equals(s.Value, Value);
        }

        public static LuaString operator +(LuaString a, LuaType b)
        {
            switch (b)
            {
                case LuaString s:
                    return a + s;
                case LuaBoolean bl:
                    return a + bl;
                case NumberLong l:
                    return a + l;
                case NumberDouble d:
                    return a + d;
            }
            throw new ArgumentException($"Adding Type '{b.Type}' to a LuaString is not supported.");
        }

        public static LuaString operator +(LuaString a, LuaString b)
        {
            return new LuaString(a.Value + b.Value);
        }

        public static LuaString operator +(LuaString a, LuaBoolean b)
        {
            return new LuaString(a.Value + b.Value);
        }

        public static LuaString operator +(LuaString a, NumberLong b)
        {
            return new LuaString(a.Value + b.Value);
        }

        public static LuaString operator +(LuaString a, NumberDouble b)
        {
            return new LuaString(a.Value + b.Value);
        }
    }
}