using System;
using System.Collections.Generic;
using System.Linq;
using Upsilon.BaseTypes;

namespace Upsilon.Binder
{
    internal class BoundScope
    {
        public readonly BoundScope ParentScope;
        internal readonly Dictionary<string, VariableSymbol> Variables;

        public BoundScope(BoundScope parentScope)
        {
            ParentScope = parentScope;
            Variables   = new Dictionary<string, VariableSymbol>();
        }
        public BoundScope(Dictionary<string, VariableSymbol> variables, BoundScope parentScope)
        {
            ParentScope = parentScope;
            Variables = variables;
        }

        public void SetVariable(VariableSymbol var)
        {
            if (Variables.ContainsKey(var.Name))
                Variables[var.Name] = var;
            else
                Variables.Add(var.Name, var);
        }

        public void SetGlobalVariable(VariableSymbol var)
        {
            if (ParentScope == null)
            {
                SetVariable(var);
            }
            else
            {
                ParentScope.SetGlobalVariable(var);
            }
        }


        public bool TryGetVariable(string key, bool allowUpperScopes, out VariableSymbol result)
        {
            if (Variables.TryGetValue(key, out result))
            {
                return true;
            }
            if (ParentScope != null && allowUpperScopes)
            {
                return ParentScope.TryGetVariable(key, true, out result);
            }
            return false;
        }

    }
}