using System;
using System.Linq;
using Upsilon.Parser;
using Type = Upsilon.BaseTypes.Type;

namespace Upsilon.Binder
{
    public class BoundUnaryOperator
    {
        public enum OperatorKind
        {
            Identity,
            Negation,
            LogicalNegation
        }

        public Type InType { get; }
        public Type OutType { get; }
        public OperatorKind Kind { get; }

        private BoundUnaryOperator(OperatorKind kind, Type inType, Type outType)
        {
            Kind = kind;
            InType = inType;
            OutType = outType;
        }

        private static BoundUnaryOperator[] _operators= new BoundUnaryOperator[]
        {
            new BoundUnaryOperator(OperatorKind.Identity, Type.Number, Type.Number),
            new BoundUnaryOperator(OperatorKind.Negation, Type.Number, Type.Number),
            new BoundUnaryOperator(OperatorKind.LogicalNegation, Type.Boolean, Type.Boolean),
        };

        public static BoundUnaryOperator Bind(SyntaxKind operatorToken, Type inType)
        {
            OperatorKind kind;
            switch (operatorToken)
            {
                case SyntaxKind.Plus:
                    kind = OperatorKind.Identity;
                    break;
                case SyntaxKind.Minus:
                    kind = OperatorKind.Negation;
                    break;
                case SyntaxKind.NotKeyword:
                    kind = OperatorKind.LogicalNegation;
                    break;
                default:
                    throw new Exception("Unknown unary operator token: " + operatorToken);
            }

            return _operators.FirstOrDefault(op => op.Kind == kind && op.InType == inType);
        }
    }
}