using System.Collections.Generic;
using Upsilon.BaseTypes;
using Upsilon.Binder;

namespace Upsilon.Evaluator
{
    internal class EvaluationScope
    {
        private readonly EvaluationScope _parentScope;
        internal readonly Dictionary<VariableSymbol, LuaType> Variables;

        internal EvaluationScope(EvaluationScope parentScope)
        {
            _parentScope = parentScope;
            Variables = new Dictionary<VariableSymbol, LuaType>();
        }
        internal EvaluationScope(Dictionary<VariableSymbol, LuaType> vars)
        {
            Variables  = vars;
        }

        public void Set(VariableSymbol symbol, LuaType obj)
        {
            if (Variables.ContainsKey(symbol))
            {
                Variables[symbol] = obj;
            }
            else
            {
                Variables.Add(symbol, obj);
            }
        }

        public void SetGlobal(VariableSymbol symbol, LuaType obj)
        {
            if (_parentScope != null)
                _parentScope.SetGlobal(symbol, obj);
            else
            {
                Set(symbol, obj);
            }
        }

        public bool TryGet(VariableSymbol symbol, out LuaType obj)
        {
            if (Variables.TryGetValue(symbol, out obj))
                return true;
            if (_parentScope != null)
                if (_parentScope.TryGet(symbol, out obj))
                    return true;
            return false;
        }

        public bool TryGet(string variable, out LuaType obj)
        {
            foreach (var luaType in Variables)
            {
                if (luaType.Key.Name == variable)
                {
                    obj = luaType.Value;
                    return true;
                };
            }
            if (_parentScope != null)
                if (_parentScope.TryGet(variable, out obj))
                    return true;
            obj = null;
            return false;
        }
    }
}