using System.Collections.Generic;
using System.Collections.Immutable;

namespace Upsilon.Parser
{
    public class FunctionStatementSyntax : StatementSyntax
    {
        public SyntaxToken LocalToken { get; }
        public SyntaxToken FunctionToken { get; }
        public IdentifierToken Identifier { get; }
        public SyntaxToken OpenParenthesis { get; }
        public ImmutableArray<IdentifierToken> Parameters { get; }
        public SyntaxToken CloseParenthesis { get; }
        public BlockStatementSyntax Block { get; }
        public SyntaxToken EndToken { get; }

        public FunctionStatementSyntax(SyntaxToken localToken, SyntaxToken functionToken, IdentifierToken identifier,
            SyntaxToken openParenthesis, ImmutableArray<IdentifierToken> parameters, SyntaxToken closeParenthesis,
            BlockStatementSyntax block, SyntaxToken endToken)
        {
            LocalToken = localToken;
            FunctionToken = functionToken;
            Identifier = identifier;
            OpenParenthesis = openParenthesis;
            Parameters = parameters;
            CloseParenthesis = closeParenthesis;
            Block = block;
            EndToken = endToken;
        }

        public override SyntaxKind Kind => SyntaxKind.FunctionStatement;
        public override IEnumerable<SyntaxNode> ChildNodes()
        {
            yield return LocalToken;
            yield return FunctionToken;
            yield return Identifier;
            yield return OpenParenthesis;
            foreach (var identifierToken in Parameters)
            {
                yield return identifierToken;
            }
            yield return CloseParenthesis;
            yield return Block;
            yield return EndToken;
        }
    }
}